// Calculating overall heat transfer coefficent
{
    // Kinematic viscosity
    volScalarField nu(thermo.nu());

    // Mean particle diameter
    dimensionedScalar dp("dp",dimensionSet(0,1,0,0,0,0,0), solidPropertiesDict.lookup("dp"));

    // Specific surface area of the bed
    dimensionedScalar SpecificSurfArea
    (
        "SpecificSurfArea",
        dimensionSet(0,-1,0,0,0,0,0), 
        solidPropertiesDict.lookup("SpecificSurfArea")
    );

    // Particle Re number
    volScalarField particleRe(mag(U)*dp/nu);

    // Laminar Prandtl number
    volScalarField Pr(rho*nu/turbulence->alpha());

    // Laminar Nusselt number
    volScalarField laminarNu = 0.664*Foam::pow(particleRe, 1.0/2.0)*Foam::pow(Pr, 1.0/3.0);

    // Turbulent Nusselt number
    volScalarField turbNu = 0.037*Foam::pow(particleRe, 0.9)*Pr
                          / (Foam::pow(particleRe, 0.1)+2.443*(Foam::pow(Pr, 2.0/3.0)-1.0));

    // Single sphere Nusselt number
    volScalarField sphereNu = 2.0+Foam::sqrt(Foam::pow(laminarNu, 2.0)+Foam::pow(turbNu, 2.0));

    // Bed shape factor
    volScalarField shapefactor(1.0+1.5*(1.0-voidFraction));

    // Calculating heat transfer coefficent
    heatTransferCoeff = shapefactor*sphereNu*turbulence->alpha()*thermo.Cp()*SpecificSurfArea/dp;

    // Setting heat transfer coefficent zero for non porous areas
    forAll(heatTransferCoeff, celli)
    {
        if(voidFraction[celli] == 1)
        {
            heatTransferCoeff[celli] = 0.0;
        }
    }
}

// Creating solid-fluid heat transfer source term
volScalarField heatTransferSource("heatTransferSource", heatTransferCoeff*(T-Tsolid));

// Solve fluid heat equation
{
    volScalarField& he = thermo.he();

    // Updating poorus density and flux
    volScalarField voidFractionRho = voidFraction*rho;
    surfaceScalarField voidFractionPhi = fvc::interpolate(voidFraction)*phi;

    fvScalarMatrix EEqn
    (
        fvm::ddt(voidFractionRho, he) + fvm::div(voidFractionPhi, he)
      + fvc::ddt(voidFractionRho, K) + fvc::div(voidFractionPhi, K)

       //Compressibity correction
      - fvm::Sp(fvc::ddt(voidFractionRho) + fvc::div(voidFractionPhi), he)
      - (fvc::ddt(voidFractionRho) + fvc::div(voidFractionPhi)) * K

      + (
            he.name() == "e"
          ? fvc::div
            (
                fvc::absolute(voidFractionPhi/fvc::interpolate(rho), U),
                p,
                "div(phiv,p)"
            )
          : -voidFraction * dpdt
        )
      - fvm::laplacian(voidFraction * turbulence->alphaEff(), he)
     ==
      - heatTransferSource
      + fvOptions(rho, he)
    );

    EEqn.relax();

    fvOptions.constrain(EEqn);

    EEqn.solve();

    fvOptions.correct(he);

    thermo.correct();
}

// Solve solid heat equation
{
    // Bed conductivity using porosity
    volScalarField kp("kp", solidConductivity/turbulence->alpha()/thermo.Cp());
    volScalarField PHI("PHI", (1.0-voidFraction)*(kp-1.0)/(kp+2.0));
    volScalarField kbed("kbed", (1.0+2.0*PHI)/(1.0-PHI));
    volScalarField bedConductivity("bedConductivity", kbed*turbulence->alpha()*thermo.Cp());
    volScalarField kappa("kappa", bedConductivity / solidDensity / solidHeatCapacity);

    // Inverse of solid void fraction
    volScalarField solidVoidFraction("solidVoidFraction", voidFraction);

    forAll(solidVoidFraction, celli)
    {
        voidFraction[celli] == 1.0 ? solidVoidFraction[celli] = 0.0:
        solidVoidFraction[celli] = 1.0 / (1.0 - voidFraction[celli]);
    }
    solidVoidFraction.correctBoundaryConditions();


    fvScalarMatrix TsEqn
    (
        fvm::ddt(Tsolid)
      - fvm::laplacian(solidVoidFraction * kappa, Tsolid)
     ==
        heatTransferSource/solidDensity/solidHeatCapacity*solidVoidFraction
      + fvOptions(Tsolid)
    );

    TsEqn.relax();

    fvOptions.constrain(TsEqn);

    TsEqn.solve();

    fvOptions.correct(Tsolid);
}