    Info<< "Reading thermophysical properties\n" << endl;

    autoPtr<psiThermo> pThermo
    (
        psiThermo::New(mesh)
    );
    psiThermo& thermo = pThermo();
    thermo.validate(args.executable(), "h", "e");

    volScalarField& p = thermo.p();
    volScalarField& T = thermo.T();
    const volScalarField& psi = thermo.psi();

    volScalarField rho
    (
        IOobject
        (
            "rho",
            runTime.timeName(),
            mesh,
            IOobject::READ_IF_PRESENT,
            IOobject::AUTO_WRITE
        ),
        thermo.rho()
    );

    Info<< "Reading field U\n" << endl;
    volVectorField U
    (
        IOobject
        (
            "U",
            runTime.timeName(),
            mesh,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        ),
        mesh
    );

    #include "compressibleCreatePhi.H"

    dimensionedScalar rhoMax(pimple.dict().lookup("rhoMax"));
    dimensionedScalar rhoMin(pimple.dict().lookup("rhoMin"));

    // Reading porous dictionary
    Info<< "Reading porous dictionary\n" << endl;
    IOdictionary porousDict
    (
        IOobject
        (
            "porousDict",
            runTime.constant(),
            mesh,
            IOobject::MUST_READ_IF_MODIFIED,
            IOobject::NO_WRITE,
            false
        )
    );

    // Selecting porous model
    word porousModel(porousDict.lookupOrDefault<word>("porousModel", "noModel"));

    // Checking for availibility of selected model
    if(porousModel != "smoothSphere")
    {
        if(porousModel == "noModel")
        {
            Info << "\nWarning: No porous model is selected!!!"
                 << "\nBy default smoothSphere model was selected\n "<< endl;
        }
        else
        {
            Info << "\nWarning: The selected porous model is not sopported!!!"
                 << "\nAvailable models are \n{ \n    smoothSphere \n}" 
                 << "\nBy default smoothSphere model was selected\n "<< endl; 
        }

        porousModel = "smoothSphere";
    }
    else
        Info << "Selecting porous model " << porousModel << "\n" << endl;

    // Checking porous resistance status
    bool porousResistance(porousDict.lookupOrDefault<bool>("porousResistance", true));

    if(porousResistance)
        Info << "Porous momentum resistance source term is enabled\n "<< endl; 
    else
        Info << "Porous momentum resistance source term is disabled\n "<< endl; 

    // Void fraction field
    Info<< "Reading field void fraction (Vfluid/Vtotal)\n" << endl;
    volScalarField voidFraction
    (
        IOobject
        (
            "voidFraction",
            runTime.timeName(),
            mesh,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        ),
        mesh
    );

    // Porous permeability
    Info<< "Creating field permeability\n" << endl;
    volScalarField permeability
    (
        IOobject
        (
            "permeability",
            runTime.timeName(),
            mesh,
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        mesh,
        dimensionedScalar("permeability",dimensionSet(0,2,0,0,0,0,0), 0.0)
    );

    // Resistance coefficent source term
    Info<< "Creating field resistanceCoef\n" << endl;
    volScalarField resistanceCoef
    (
        IOobject
        (
            "resistanceCoef",
            runTime.timeName(),
            mesh,
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        mesh,
        dimensionedScalar("resistanceCoef",dimensionSet(1,-3,-1,0,0,0,0), 0.0)
    );

    // Solid temperature field
    Info<< "Reading field Tsolid\n" << endl;
    volScalarField Tsolid
    (
        IOobject
        (
            "Tsolid",
            runTime.timeName(),
            mesh,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        ),
        mesh
    );

    // Creating heat transfer coefficents field
    Info<< "Creating field heatTransferCoeff\n" << endl;
    volScalarField heatTransferCoeff
    (
        IOobject
        (
            "heatTransferCoeff",
            runTime.timeName(),
            mesh
        ),
        mesh,
        dimensionedScalar("heatTransferCoeff", dimEnergy/dimVolume/dimTemperature/dimTime, 0.0)
    );

    // Reading solid matrix properties
    dictionary solidPropertiesDict(porousDict.subDict("solidProperties"));
    
    dimensionedScalar solidDensity
    (
        "solidDensity", 
        dimensionSet(1,-3,0,0,0,0,0),
        solidPropertiesDict.lookup("solidDensity")
    );
    dimensionedScalar solidConductivity
    (
        "solidConductivity", 
        dimensionSet(1,1,-3,-1,0,0,0),
        solidPropertiesDict.lookup("solidConductivity")
    );
    dimensionedScalar solidHeatCapacity
    (
        "solidHeatCapacity", 
        dimensionSet(0,2,-2,-1,0,0,0),
        solidPropertiesDict.lookup("solidHeatCapacity")
    );


    Info<< "Creating turbulence model\n" << endl;
    autoPtr<compressible::turbulenceModel> turbulence
    (
        compressible::turbulenceModel::New
        (
            rho,
            U,
            phi,
            thermo
        )
    );

    Info<< "Creating field dpdt\n" << endl;
    volScalarField dpdt
    (
        IOobject
        (
            "dpdt",
            runTime.timeName(),
            mesh
        ),
        mesh,
        dimensionedScalar("dpdt", p.dimensions()/dimTime, 0)
    );

    Info<< "Creating field kinetic energy K\n" << endl;
    volScalarField K("K", 0.5*magSqr(U));